local listSkeletalSets = {}

local function dilate(imgA, imgB)
	local imgOut = {}
	for y, row in ipairs(imgA) do
		imgOut[y] = {}
		for x, val in ipairs(row) do
			imgOut[y][x] = 0
		end
	end

	for y, row in ipairs(imgA) do
		for x, val in ipairs(row) do
			if val == 1 then
				local h = #imgB
				local w = #imgB[1]

				local centerX = math.ceil(w/2)
				local centerY = math.ceil(h/2)
				for yB, rowB in ipairs(imgB) do
					for xB, valB in ipairs(rowB) do
						local testX = x - centerX + xB
						local testY = y - centerY + yB
						if (imgOut[testY]) then
							if (imgOut[testY][testX] == 0) then
								imgOut[testY][testX] = imgB[yB][xB]
							end
						end
					end
				end
			end
		end
	end
	return imgOut
end



local function erode(imgA, imgB)
	local imgOut = {}
	for y, row in ipairs(imgA) do
		imgOut[y] = {}
		for x, val in ipairs(row) do
			imgOut[y][x] = 0
		end
	end

	for y, row in ipairs(imgA) do
		for x, val in ipairs(row) do
			if val == 1 then
				imgOut[y][x] = 1
				local h = #imgB
				local w = #imgB[1]

				local centerX = math.ceil(w/2)
				local centerY = math.ceil(h/2)
				for yB, rowB in ipairs(imgB) do
					for xB, valB in ipairs(rowB) do
						local testX = x - centerX + xB
						local testY = y - centerY + yB
						if (imgOut[testY]) and (imgOut[testY][testX]) then
							--print(testX, testY, imgOut[testY][testX])
							if not (imgA[testY][testX] >= imgB[yB][xB]) then
								imgOut[y][x] = 0
							end
						elseif (imgB[yB][xB] == 1) then
							imgOut[y][x] = 0
						end
					end
				end
			end
		end
	end
	return imgOut
end

local function open(imgA, imgB)
	return dilate(erode(imgA, imgB), imgB)

end

--assumes same size
local function difference(imgA, imgB)
	imgOut = {}
	for y, row in ipairs(imgA) do
		imgOut[y] = {}
		for x, val in ipairs(row) do
			if (imgA[y][x] == imgB[y][x]) then
				imgOut[y][x] = 0
			else
				imgOut[y][x] = imgA[y][x] 
			end
		end
	end 
	return imgOut
end

local function toBinary(imgData)
	local h = imgData:getHeight()
	local w = imgData:getWidth()
	local img = {}
	for y=1, h do
		img[y] = {}
		for x=1, w do
			local r,g,b = imgData:getPixel(x-1,y-1)
			if r < .5 and g <.5 and b  <.5 then
				img[y][x] = 1
			else
				img[y][x] = 0
			end
		end
	end
	return img
end

local function deepcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value)
        end
        setmetatable(copy, deepcopy(getmetatable(orig)))
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

local function generateB(n)
	local unit = {}
	for y=1, 65 do
		unit[y] = {}
		for x=1, 65 do
			if n > 0 then
				if (x == 33 and y == 33) or (x == 32 and y == 33) or (x == 34 and y == 33) 
					or (x == 33 and y == 32) or (x == 33 and y == 34) then			
					unit[y][x] = 1				
				else
					unit[y][x] = 0
				end
			else
				if(x == 33 and y == 33) then
					unit[y][x] = 1
				end
			end
		end
	end

	local b = deepcopy(unit)
	local i = 1
	while (i < n ) do
		b = dilate(b,unit)
		i = i + 1
	end
	return b
end

local function generateSkeletalSet(img, b)
	local unit = {}
	for y=1, 65 do
		unit[y] = {}
		for x=1, 65 do

			if (x == 33 and y == 33) or (x == 32 and y == 33) or (x == 34 and y == 33) 
				or (x == 33 and y == 32) or (x == 33 and y == 34) then			
				unit[y][x] = 1				
			else
				unit[y][x] = 0
			end
		end
	end

	local part1 = erode(img, b)
	local part2 = open(part1, unit)

	return difference(part1, part2)
end

local function union(imgA, imgB)
	imgOut = {}
	for y, row in ipairs(imgA) do
		imgOut[y] = {}
		for x, val in ipairs(row) do
			if (imgA[y][x] == 1 or imgB[y][x] == 1) then
				imgOut[y][x] = 1
			else
				imgOut[y][x] = 0 
			end
		end
	end 
	return imgOut
end

local function checkEmpty(img)
	for y, row in ipairs(img) do
		for x, val in ipairs(row) do
			if val == 1 then
				return false
			end
		end
	end
	return true
end

function  love.load( ... )
	love.graphics.setBackgroundColor(1,0,1,1)
	deer = toBinary(love.image.newImageData( "deer.png" ))

	local i = -1
	local b0 = generateB(0)
	local unit = generateB(1)
	local b = generateB(1)

	repeat 
		i = i + 1
		if i == 0 then
			listSkeletalSets[0] = generateSkeletalSet(deer, b0)
		else
			listSkeletalSets[i] = generateSkeletalSet(deer, b)
			b = dilate(b,unit)
		end
	until checkEmpty(listSkeletalSets[i])

	local b = generateB(1)	

	for i = 0, #listSkeletalSets do
		if img then
			img = union(img, listSkeletalSets[i])
			b = dilate(b,unit)
		else
			if i == 0 then
				img = listSkeletalSets[0]
			else
				b = generateB(i)
				img = union(listSkeletalSets[i])
			end
		end
			
	end

end

function love.draw()
	for y,row in ipairs(img) do
		for x,val in ipairs(row) do
			if val == 0 then
				love.graphics.setColor(1,1,1,1)
				love.graphics.rectangle("fill", 5*x, 5*y, 5,5)
			else	
				love.graphics.setColor(0,0,0,0)
				love.graphics.rectangle("fill", 5*x, 5*y, 5,5)
			end
		end
	end
end


